@@ -1,7 +1,7 @@
 #
 #	Gnu.pm --- The GNU Readline/History Library wrapper module
 #
-#	$Id: Gnu.pm 476 2014-12-13 03:56:06Z hayashi $
+#	$Id: Gnu.pm 481 2015-01-31 12:07:49Z hayashi $
 #
 #	Copyright (c) 1996 Hiroo Hayashi.  All rights reserved.
 #
@@ -82,7 +82,7 @@ END
     use DynaLoader;
     use vars qw($VERSION @ISA @EXPORT_OK);
 
-    $VERSION = '1.25';		# update Gnu::XS::VERSION also.
+    $VERSION = '1.26';		# update Gnu::XS::VERSION also.
 
     # Term::ReadLine::Gnu::AU makes a function in
     # `Term::ReadLine::Gnu::XS' as a method.
@@ -246,9 +246,9 @@ sub new {
     # calls setenv() before the 1st assignment to $ENV{}.
     $ENV{_TRL_DUMMY} = '';
 
-    # enable ornaments to be compatible with perl5.004_05(?)
-    $self->ornaments(1) unless ($ENV{PERL_RL} and $ENV{PERL_RL} =~ /\bo\w*=0/);
-
+    # set tty before calling rl_initialize() not to output some
+    # charactores to STDIO.
+    # https://rt.cpan.org/Ticket/Display.html?id=96569
     if (!@_) {
 	my ($IN,$OUT) = $self->findConsole();
 	open(IN,"<$IN")   || croak "Cannot open $IN for read";
@@ -263,13 +263,17 @@ sub new {
 	$Attribs{instream} = shift;
 	$Attribs{outstream} = shift;
     }
-    $readline_version = $Attribs{readline_version};
 
     # initialize the GNU Readline Library and termcap library
-    # This is not necessary but is left to keep backward compatibility.
-    # https://rt.cpan.org/Ticket/Display.html?id=96569
+    # This calls tgetent().
     $self->initialize();
 
+    # enable ornaments to be compatible with perl5.004_05(?)
+    # This calls tgetstr().
+    $self->ornaments(1) unless ($ENV{PERL_RL} and $ENV{PERL_RL} =~ /\bo\w*=0/);
+
+    $readline_version = $Attribs{readline_version};
+
     $self;
 }
 
@@ -656,6 +660,8 @@ use vars qw(%_rl_vars);
        rl_last_func                             => ['LF', 0],
       );
 
+my @stream;
+
 sub TIESCALAR {
     my $class = shift;
     my $name = shift;
@@ -682,7 +688,8 @@ sub FETCH {
     } elsif ($type eq 'F') {
 	return _rl_fetch_function($id);
     } elsif ($type eq 'IO') {
-	return _rl_fetch_iostream($id);
+	# STORE was called in new() before coming here
+	return $stream[$id];
     } elsif ($type eq 'K') {
 	return _rl_fetch_keymap($id);
     } elsif ($type eq 'LF') {
@@ -718,14 +725,16 @@ sub STORE {
     } elsif ($type eq 'F') {
 	return _rl_store_function($value, $id);
     } elsif ($type eq 'IO') {
+	my $FH = $value;
+	# Pass filehandles to the GNU Readline Library
+	_rl_store_iostream($FH, $id);
 	# pop stdio layer pushed by PerlIO_findFILE().
 	# https://rt.cpan.org/Ticket/Display.html?id=59832
-	my $FH = _rl_store_iostream($value, $id);
 	my @layers = PerlIO::get_layers($FH);
 	#warn "$id<", join(':', @layers), "\n";
 	binmode($FH, ":pop") if @layers > 1;
 	#@layers = PerlIO::get_layers($FH); warn "$id>", join(':', @layers), "\n";
-	return $FH;
+	return $stream[$id] = $FH;
     } elsif ($type eq 'K' || $type eq 'LF') {
 	carp "Term::ReadLine::Gnu::Var::STORE: read only variable `$name'\n";
 	return undef;
